@@ -1,11 +1,13 @@
-from json import JSONEncoder
-from datetime import datetime
+"""Structure of a standard EPG prorgramme."""
 
 
 class Programme:
+    """SkyQ Programme Class."""
+
     def __init__(
         self, programmeuuid, starttime, endtime, title, season, episode, imageUrl
     ):
+        """Programme structure for SkyQ."""
         self.progammeuuid = programmeuuid
         self.starttime = starttime
         self.endtime = endtime
