@@ -5,14 +5,13 @@ import requests
 
 from pyskyqremote.channel import Programme
 
-RESPONSE_OK = 200
-
-SCHEDULE_URL = "https://apid.sky.it/gtv/v1/events?from={1}&to={2}&pageSize=50&pageNum=0&env=DTH&channels={0}"
-LIVE_IMAGE_URL = "https://ethaneurope.it.imageservice.sky.com/pd-image/{0}/16-9"
-PVR_IMAGE_URL = "https://ethaneurope.it.imageservice.sky.com/pd-image/{0}/16-9"
-CHANNEL_URL = "https://apid.sky.it/gtv/v1/channels?env=DTH"
-CHANNEL_IMAGE_URL = (
-    "https://ethaneurope.it.imageservice.sky.com/pd-logo/skychb_{0}{1}/600/600"
+from ..const import RESPONSE_OK
+from .const_it import (
+    CHANNEL_IMAGE_URL,
+    PVR_IMAGE_URL,
+    SCHEDULE_URL,
+    LIVE_IMAGE_URL,
+    CHANNEL_URL,
 )
 
 _LOGGER = logging.getLogger(__name__)
