@@ -6,229 +6,119 @@ import requests
 import json
 import xmltodict
 import logging
+import importlib
+import pycountry
 from datetime import datetime, timedelta
 from http import HTTPStatus
 
 from ws4py.client.threadedclient import WebSocketClient
 
+from .const import (
+    SKY_PLAY_URN,
+    SKYCONTROL,
+    SOAP_ACTION,
+    SOAP_CONTROL_BASE_URL,
+    SOAP_DESCRIPTION_BASE_URL,
+    SOAP_PAYLOAD,
+    SOAP_RESPONSE,
+    SOAP_USER_AGENT,
+    UPNP_GET_MEDIA_INFO,
+    UPNP_GET_TRANSPORT_INFO,
+    WS_BASE_URL,
+    WS_CURRENT_APPS,
+    REST_BASE_URL,
+    REST_CHANNEL_LIST,
+    REST_RECORDING_DETAILS,
+    REST_PATH_INFO,
+    REST_PATH_DEVICEINFO,
+    DEFAULT_ENCODING,
+    CURRENT_URI,
+    CURRENT_TRANSPORT_STATE,
+    APP_STATUS_VISIBLE,
+    PVR,
+    XSI,
+    PAST_END_OF_EPG,
+    CONNECTTIMEOUT,
+    TIMEOUT,
+    COMMANDS,
+    SKY_STATE_PLAYING,
+    SKY_STATE_PAUSED,
+    SKY_STATE_STANDBY,
+    SKY_STATE_ON,
+    SKY_STATE_OFF,
+    APP_EPG,
+)
+from .const import TEST_CHANNEL_LIST
+
 _LOGGER = logging.getLogger(__name__)
 
-# SOAP/UPnP Constants
-SKY_PLAY_URN = "urn:nds-com:serviceId:SkyPlay"
-SKYCONTROL = "SkyControl"
-SOAP_ACTION = '"urn:schemas-nds-com:service:SkyPlay:2#{0}"'
-SOAP_CONTROL_BASE_URL = "http://{0}:49153{1}"
-SOAP_DESCRIPTION_BASE_URL = "http://{0}:49153/description{1}.xml"
-SOAP_PAYLOAD = """<s:Envelope xmlns:s='http://schemas.xmlsoap.org/soap/envelope/' s:encodingStyle='http://schemas.xmlsoap.org/soap/encoding/'>
-    <s:Body>
-        <u:{0} xmlns:u="urn:schemas-nds-com:service:SkyPlay:2">
-            <InstanceID>0</InstanceID>
-        </u:{0}>
-    </s:Body>
-</s:Envelope>"""
-SOAP_RESPONSE = "u:{0}Response"
-SOAP_USER_AGENT = "SKYPLUS_skyplus"
-UPNP_GET_MEDIA_INFO = "GetMediaInfo"
-UPNP_GET_TRANSPORT_INFO = "GetTransportInfo"
-
-# WebSocket Constants
-WS_BASE_URL = "ws://{0}:9006/as/{1}"
-WS_CURRENT_APPS = "apps/status"
-
-# REST Constants
-REST_BASE_URL = "http://{0}:{1}/as/{2}"
-REST_CHANNEL_LIST = "services"
-REST_RECORDING_DETAILS = "pvr/details/{0}"
-REST_PATH_INFO = "system/information"
-
-# Generic Constants
-DEFAULT_ENCODING = "utf-8"
-
-# Sky specific constants
-CURRENT_URI = "CurrentURI"
-CURRENT_TRANSPORT_STATE = "CurrentTransportState"
-APP_STATUS_VISIBLE = "VISIBLE"
-
-PVR = "pvr"
-XSI = "xsi"
-PAST_END_OF_EPG = "past end of epg"
-
-CONNECTTIMEOUT = 1000
-TIMEOUT = 2
-
-
-TEST_CHANNEL_LIST = {
-    "documentId": "2416",
-    "services": [
-        {
-            "c": "317",
-            "dvbtriplet": "64511.8800.11684",
-            "schedule": True,
-            "servicetype": "DSAT",
-            "servicetypes": ["DSAT"],
-            "sf": "hd",
-            "sg": 12,
-            "sid": "684",
-            "sk": 684,
-            "t": "Premium Comedy HD",
-            "xsg": 3,
-        },
-        {
-            "c": "400",
-            "dvbtriplet": "64511.6400.11075",
-            "schedule": True,
-            "servicetype": "DSAT",
-            "servicetypes": ["DSAT"],
-            "sf": "hd",
-            "sg": 13,
-            "sid": "75",
-            "sk": 75,
-            "t": "Sky Arte HD",
-            "xsg": 4,
-        },
-        {
-            "c": "120",
-            "dvbtriplet": "64511.6400.11074",
-            "schedule": True,
-            "servicetype": "DSAT",
-            "servicetypes": ["DSAT"],
-            "sf": "hd",
-            "sg": 21,
-            "sid": "74",
-            "sk": 74,
-            "t": "Sky Arte HD",
-            "xsg": 1,
-        },
-    ],
-}
-
 
 class SkyQRemote:
     """SkyQRemote is the instantiation of the SKYQ remote ccontrol."""
 
-    commands = {
-        "power": 0,
-        "select": 1,
-        "backup": 2,
-        "dismiss": 2,
-        "channelup": 6,
-        "channeldown": 7,
-        "interactive": 8,
-        "sidebar": 8,
-        "help": 9,
-        "services": 10,
-        "search": 10,
-        "tvguide": 11,
-        "home": 11,
-        "i": 14,
-        "text": 15,
-        "up": 16,
-        "down": 17,
-        "left": 18,
-        "right": 19,
-        "red": 32,
-        "green": 33,
-        "yellow": 34,
-        "blue": 35,
-        "0": 48,
-        "1": 49,
-        "2": 50,
-        "3": 51,
-        "4": 52,
-        "5": 53,
-        "6": 54,
-        "7": 55,
-        "8": 56,
-        "9": 57,
-        "play": 64,
-        "pause": 65,
-        "stop": 66,
-        "record": 67,
-        "fastforward": 69,
-        "rewind": 71,
-        "boxoffice": 240,
-        "sky": 241,
-    }
+    commands = COMMANDS
 
-    SKY_STATE_PLAYING = "PLAYING"
-    SKY_STATE_PAUSED = "PAUSED_PLAYBACK"
-    SKY_STATE_OFF = "OFF"
-    SKY_STATE_ON = "ON"
-    SKY_STATE_POWERED_OFF = "POWERED OFF"
-
-    # Application Constants
-    APP_EPG = "com.bskyb.epgui"
-
-    def __init__(self, host, country, port=49160, jsonport=9006):
+    def __init__(
+        self, host, overrideCountry=None, test_channel=None, port=49160, jsonport=9006,
+    ):
         """Stand up a new SkyQ box."""
         self._host = host
-        self._country = country.casefold()
+        self._test_channel = test_channel
         self._port = port
         self._jsonport = jsonport
-
-        url_index = 0
+        self._overrideCountry = overrideCountry
+        self.deviceSetup = False
         self._soapControlURL = None
-        while self._soapControlURL is None and url_index < 50:
-            self._soapControlURL = self._getSoapControlURL(url_index)["url"]
-            url_index += 1
 
-        if self._country == "uk":
-            from pyskyqremote.country.remote_uk import SkyQCountry
-        elif self._country == "it":
-            from pyskyqremote.country.remote_it import SkyQCountry
-        elif self._country == "test":
-            from pyskyqremote.country.remote_it import SkyQCountry
-        else:
-            _LOGGER.exception(
-                f"X0999 - Invalid country: {self._host} : {self._country}"
+        self.powerStatus()
+        if not self.deviceSetup:
+            _LOGGER.error(
+                f"E0060 - Device not switched on during restart: {self._host}"
             )
 
-        self._remoteCountry = SkyQCountry(self._host)
-        if self._country != "test":
-            self._channels = self._http_json(REST_CHANNEL_LIST)
-        else:
-            self._channels = TEST_CHANNEL_LIST
-
     def powerStatus(self) -> str:
         """Get the power status of the Sky Q box."""
+        if not self.deviceSetup:
+            self._setupDevice()
+
         if self._soapControlURL is None:
-            return self.SKY_STATE_POWERED_OFF
+            return SKY_STATE_OFF
         try:
             output = self._http_json(REST_PATH_INFO)
             if "activeStandby" in output and output["activeStandby"] is False:
-                return self.SKY_STATE_ON
-            return self.SKY_STATE_OFF
+                return SKY_STATE_ON
+            return SKY_STATE_STANDBY
         except (
             requests.exceptions.ConnectTimeout,
             requests.exceptions.ReadTimeout,
         ):
-            return self.SKY_STATE_OFF
+            return SKY_STATE_STANDBY
         except (requests.exceptions.ConnectionError):
             _LOGGER.info(
                 f"I0010 - Device has control URL but connection request failed: {self._host}"
             )
-            return self.SKY_STATE_OFF
+            return SKY_STATE_OFF
         except Exception as err:
             _LOGGER.exception(f"X0060 - Error occurred: {self._host} : {err}")
-            return self.SKY_STATE_OFF
+            return SKY_STATE_STANDBY
 
     def getCurrentState(self):
         """Get current state of the SkyQ box."""
-        if self.powerStatus() == self.SKY_STATE_OFF:
-            return self.SKY_STATE_OFF
+        if self.powerStatus() == SKY_STATE_STANDBY:
+            return SKY_STATE_STANDBY
         response = self._callSkySOAPService(UPNP_GET_TRANSPORT_INFO)
         if response is not None:
             state = response[CURRENT_TRANSPORT_STATE]
-            if state == self.SKY_STATE_PLAYING:
-                return self.SKY_STATE_PLAYING
-            if state == self.SKY_STATE_PAUSED:
-                return self.SKY_STATE_PAUSED
-        return self.SKY_STATE_OFF
+            if state == SKY_STATE_PLAYING:
+                return SKY_STATE_PLAYING
+            if state == SKY_STATE_PAUSED:
+                return SKY_STATE_PAUSED
+        return SKY_STATE_STANDBY
 
     def getActiveApplication(self):
         """Get the active application on Sky Q box."""
         try:
-            result = self.APP_EPG
+            result = APP_EPG
             apps = self._callSkyWebSocket(WS_CURRENT_APPS)
             if apps is None:
                 return result
@@ -247,10 +137,8 @@ class SkyQRemote:
         result = {
             "channel": None,
             "imageUrl": None,
-            "title": None,
-            "season": None,
-            "episode": None,
             "sid": None,
+            "pvrId": None,
             "live": False,
         }
         response = self._callSkySOAPService(UPNP_GET_MEDIA_INFO)
@@ -261,8 +149,8 @@ class SkyQRemote:
                     # Live content
                     sid = int(currentURI[6:], 16)
 
-                    if self._country == "test":
-                        sid = "684"  # 5435
+                    if self._test_channel:
+                        sid = self._test_channel
 
                     channel = self._getChannelNode(sid)["channel"]
                     result.update({"sid": sid, "live": True})
@@ -272,27 +160,9 @@ class SkyQRemote:
                 elif PVR in currentURI:
                     # Recorded content
                     pvrId = "P" + currentURI[11:]
-                    recording = self._http_json(REST_RECORDING_DETAILS.format(pvrId))
-                    result.update({"channel": recording["details"]["cn"]})
-                    result.update({"title": recording["details"]["t"]})
-                    if (
-                        "seasonnumber" in recording["details"]
-                        and "episodenumber" in recording["details"]
-                    ):
-                        result.update({"season": recording["details"]["seasonnumber"]})
-                        result.update(
-                            {"episode": recording["details"]["episodenumber"]}
-                        )
-                    if "programmeuuid" in recording["details"]:
-                        programmeuuid = recording["details"]["programmeuuid"]
-                        imageUrl = self._remoteCountry.pvr_image_url.format(
-                            str(programmeuuid)
-                        )
-                        if "osid" in recording["details"]:
-                            osid = recording["details"]["osid"]
-                            imageUrl += "?sid=" + str(osid)
-                        result.update({"imageUrl": imageUrl})
-        return result
+                    result.update({"pvrId": pvrId, "live": False})
+
+        return _objectview(result)
 
     def getEpgData(self, sid, epgDate):
         """Get EPG data for the specified channel."""
@@ -332,10 +202,36 @@ class SkyQRemote:
             result.update({"episode": programme["episode"]})
             result.update({"season": programme["season"]})
             result.update({"imageUrl": programme["imageUrl"]})
-            return result
+            return _objectview(result)
         except Exception as err:
             _LOGGER.exception(f"X0030 - Error occurred: {self._host} : {sid} : {err}")
-            return result
+            return _objectview(result)
+
+    def getRecording(self, pvrId):
+        """Get the recording details."""
+        result = {
+            "channel": None,
+            "imageUrl": None,
+            "title": None,
+            "season": None,
+            "episode": None,
+        }
+
+        recording = self._http_json(REST_RECORDING_DETAILS.format(pvrId))
+        result.update({"channel": recording["details"]["cn"]})
+        result.update({"title": recording["details"]["t"]})
+        if (
+            "seasonnumber" in recording["details"]
+            and "episodenumber" in recording["details"]
+        ):
+            result.update({"season": recording["details"]["seasonnumber"]})
+            result.update({"episode": recording["details"]["episodenumber"]})
+        if "programmeuuid" in recording["details"]:
+            programmeuuid = recording["details"]["programmeuuid"]
+            imageUrl = self._remoteCountry.pvr_image_url.format(str(programmeuuid))
+            result.update({"imageUrl": imageUrl})
+
+        return _objectview(result)
 
     def press(self, sequence):
         """Issue the specified sequence of commands to SkyQ box."""
@@ -508,6 +404,58 @@ class SkyQRemote:
         channelno = channelNode["c"]
         return {"channel": channel, "channelno": channelno}
 
+    def _getDeviceInformation(self):
+        try:
+            resp = self._http_json(REST_PATH_DEVICEINFO)
+            return resp
+        except requests.exceptions.ConnectTimeout:
+            return None
+        except Exception as err:
+            _LOGGER.exception(f"X0080 - Error occurred: {self._host} : {err}")
+            return None
+
+    def _setupDevice(self):
+        """Set the remote up."""
+        deviceInfo = self._getDeviceInformation()
+        if not deviceInfo:
+            return None
+
+        alpha3 = None
+        if self._overrideCountry:
+            alpha3 = self._overrideCountry
+        elif "countryCode" in deviceInfo:
+            alpha3 = deviceInfo["countryCode"].upper()
+
+        if not alpha3:
+            _LOGGER.error(f"E0050 - No country identified: {self._host}")
+            return None
+
+        country = pycountry.countries.get(alpha_3=alpha3).alpha_2.casefold()
+
+        url_index = 0
+        self._soapControlURL = None
+        while self._soapControlURL is None and url_index < 50:
+            self._soapControlURL = self._getSoapControlURL(url_index)["url"]
+            url_index += 1
+
+        try:
+            SkyQCountry = importlib.import_module(
+                "pyskyqremote.country.remote_" + country
+            ).SkyQCountry
+
+            self._remoteCountry = SkyQCountry(self._host)
+            if not self._test_channel:
+                self._channels = self._http_json(REST_CHANNEL_LIST)
+            else:
+                self._channels = TEST_CHANNEL_LIST
+
+            self.deviceSetup = True
+            return "ok"
+
+        except Exception:
+            _LOGGER.error(f"E0040 - Invalid country: {self._host} : {alpha3}")
+            return None
+
 
 class _SkyWebSocket(WebSocketClient):
     def __init__(self, url):
@@ -516,3 +464,8 @@ class _SkyWebSocket(WebSocketClient):
 
     def received_message(self, message):
         self.data = message.data
+
+
+class _objectview(object):
+    def __init__(self, d):
+        self.__dict__ = d
